<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * @author    Presta.Site
 * @copyright 2020 Presta.Site
 * @license   LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MultiCurrency extends Module
{
    protected $html;

    public $settings_prefix = 'MCUR_';

    public $front_currency;
    public $ps_currency_default;

    public function __construct()
    {
        $this->name = 'multicurrency';
        $this->tab = 'administration';
        $this->version = '1.1.0';
        $this->ps_versions_compliancy = ['min' => '1.6.1.0', 'max' => _PS_VERSION_];
        $this->author = 'PrestaSite';
        $this->bootstrap = true;
        $this->module_key = '';

        parent::__construct();

        if ($this->active) {
            $this->loadSettings();
        }

        $this->displayName = $this->l('Multi Currency');
        $this->description = $this->l('Module for advanced currency management.');
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        // Register hooks
        $this->installHooks();

        // Create tables
        $this->installDB();

        // default values:
        $this->installDefaultSettings();

        // Generate settings
        $this->loadSettings();

        return true;
    }

    public function installHooks()
    {
        $hooks = ['actionAdminControllerSetMedia'];

        foreach ($hooks as $hook) {
            $this->registerHook($hook);
        }

        return true;
    }

    protected function installDB()
    {
        $install_queries = $this->getDbTables();
        foreach ($install_queries as $query) {
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }

        return true;
    }

    protected function getDbTables()
    {
        return [];
    }

    protected function installDefaultSettings()
    {
        foreach ($this->getSettings() as $item) {
            if ($item['type'] == 'html') {
                continue;
            }
            $item_name = Tools::strtoupper($item['name']);
            if (isset($item['default']) && (Configuration::get($this->settings_prefix . $item_name) === false)) {
                if (isset($item['lang']) && $item['lang']) {
                    $lang_value = [];
                    $set = false;
                    foreach (Language::getLanguages() as $lang) {
                        $lang_value[$lang['id_lang']] = $item['default'];
                        if (Configuration::get($this->settings_prefix . $item_name, $lang['id_lang']) !== false) {
                            $set = true;
                        }
                    }
                    if (!$set && sizeof($lang_value)) {
                        Configuration::updateValue($this->settings_prefix . $item_name, $lang_value, true);
                    }
                } else {
                    Configuration::updateValue($this->settings_prefix . $item_name, $item['default']);
                }
            }
        }
    }

    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }

        // drop tables
        foreach ($this->getDbTables() as $table_name => $query) {
            Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . pSQL($table_name) . '`;');
        }
        // reset settings
        Db::getInstance()->execute(
            'DELETE FROM `' . _DB_PREFIX_ . 'configuration` WHERE `name` LIKE "' . pSQL($this->settings_prefix) . '%";'
        );

        return true;
    }

    public function reset()
    {
        return $this->uninstall() && $this->install();
    }

    protected function loadSettings()
    {
        foreach ($this->getSettings() as $item) {
            if ($item['type'] == 'html') {
                continue;
            }

            $name = Tools::strtolower($item['name']);
            if (isset($item['lang']) && $item['lang']) {
                $this->$name = [];
                foreach (Language::getLanguages() as $language) {
                    $this->{$name}[$language['id_lang']] = Configuration::get(
                        $this->settings_prefix . $item['name'],
                        $language['id_lang']
                    );
                }
            } else {
                $this->$name = Configuration::get(
                    $this->settings_prefix .
                    $item['name']
                );
            }
        }
    }

    public function getContent()
    {
        $this->html = '';
        $this->html .= $this->renderDevInfo();
        $this->html .= $this->postProcess();

        if (!$this->active) {
            return $this->displayError($this->l('The module is deactivated'));
        }

        $this->html .= $this->renderForm();

        return $this->html;
    }

    protected function renderDevInfo()
    {
        $this->context->smarty->assign([
            'img_dir' => $this->_path . 'views/img/',
        ]);

        return $this->context->smarty->fetch($this->local_path . 'views/templates/admin/devby.tpl');
    }

    protected function postProcess()
    {
        $html = '';
        $settings_updated = false;
        $errors = [];
        $conf = 6;

        // Check if this is an ajax call / PS1.5
        if ($this->getPSVersion() < 1.6 && Tools::getIsset('ajax')
            && Tools::getValue('ajax') && Tools::getValue('action')) {
            if (is_callable([$this, 'ajaxProcess' . Tools::getValue('action')])) {
                call_user_func([$this, 'ajaxProcess' . Tools::getValue('action')]);
            }
            exit;
        }

        if (Tools::isSubmit('submitModule')) {
            // saving settings:
            $settings = $this->getSettings();
            foreach ($settings as $item) {
                if ($item['type'] == 'html' || (isset($item['lang']) && $item['lang'] == true)) {
                    continue;
                }
                if (Tools::isSubmit($item['name'])) {
                    $validated = true;
                    $val_method = (isset($item['validate']) ? $item['validate'] : '');

                    if (Tools::strlen(Tools::getValue($item['name']))) {
                        // Validation:
                        if (Tools::strlen($val_method) && is_callable(['Validate', $val_method])) {
                            $validated =
                                call_user_func(['Validate', $val_method], Tools::getValue($item['name']));
                        }
                    }
                    if ($validated) {
                        Configuration::updateValue(
                            $this->settings_prefix . $item['name'],
                            Tools::getValue($item['name']),
                            true
                        );
                        $settings_updated = true;
                    } else {
                        $label = trim($item['label'], ':');
                        $errors[] = sprintf($this->l('The "%s" field is invalid'), $label);
                    }
                }
            }

            // update lang fields:
            $languages = Language::getLanguages();
            foreach ($settings as $item) {
                if (!(isset($item['lang']) && $item['lang'])) {
                    continue;
                }
                $val_method = (isset($item['validate']) ? $item['validate'] : '');
                $lang_value = [];
                foreach ($languages as $lang) {
                    if (Tools::isSubmit($item['name'] . '_' . $lang['id_lang'])) {
                        $validated = true;
                        if (Tools::strlen(Tools::getValue($item['name'] . '_' . $lang['id_lang']))) {
                            // Validation:
                            if (Tools::strlen($val_method) && is_callable(['Validate', $val_method])) {
                                $validated = call_user_func(
                                    ['Validate', $val_method],
                                    Tools::getValue($item['name'] . '_' . $lang['id_lang'])
                                );
                            }
                        }
                        if ($validated) {
                            $lang_value[$lang['id_lang']] = Tools::getValue($item['name'] . '_' . $lang['id_lang']);
                            $settings_updated = true;
                        } else {
                            $label = trim($item['label'], ':');
                            $errors[] = sprintf($this->l('The "%s" field is invalid'), $label);
                        }
                    }
                }
                if (sizeof($lang_value)) {
                    Configuration::updateValue($this->settings_prefix . $item['name'], $lang_value, true);
                }
            }
        }

        $this->loadSettings();

        // refresh rates after saving settings:
        if ($settings_updated || Tools::getValue('convert_prices')) {
            if ($this->front_currency == $this->ps_currency_default) {
                $currency = new Currency($this->ps_currency_default);
                $currency->conversion_rate = 1;
                $currency->save();
            }

            if ($this->ps_currency_default) {
                $default_currency = new Currency($this->ps_currency_default);
                if (Validate::isLoadedObject($default_currency) && $default_currency->active) {
                    Configuration::updateValue('PS_CURRENCY_DEFAULT', (int)$this->ps_currency_default);
                } else {
                    $errors[] = $this->l('Unable to change the back office currency');
                }
            }

            $this->refreshRatesAndPrices();
        }

        if ($settings_updated && !sizeof($errors)) {
            Configuration::updateValue($this->settings_prefix . 'SETTINGS_UPDATED', time());
            $token = Tools::getAdminTokenLite('AdminModules');
            $redirect_url = 'index.php?tab=AdminModules&configure=' . $this->name . '&token=' . $token . '&conf=' . $conf;
            Tools::redirectAdmin($redirect_url);
        } elseif (sizeof($errors)) {
            foreach ($errors as $err) {
                $html .= $this->displayError($err);
            }
        }

        return $html;
    }

    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang =
            Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ?
                Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') :
                0;

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => [],
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        $helper->module = $this;

        $settings = $this->getSettings();
        $field_forms = [
            [
                'form' => [
                    'legend' => [
                        'title' => $this->l('Settings'),
                        'icon' => 'icon-cogs',
                    ],
                    'input' => $settings,
                    'submit' => [
                        'title' => $this->l('Save'),
                    ],
                ],
            ],
        ];

        foreach ($field_forms as &$field_form) {
            if ($this->getPSVersion() == 1.5) {
                $field_form['form']['submit']['class'] = 'button';
            }
        }

        foreach ($settings as $item) {
            if ($item['type'] == 'html') {
                continue;
            }
            if (isset($item['lang']) && $item['lang']) {
                foreach (Language::getLanguages() as $language) {
                    $helper->tpl_vars['fields_value'][$item['name']][$language['id_lang']] = Configuration::get(
                        $this->settings_prefix . $item['name'],
                        $language['id_lang']
                    );
                }
            } else {
                $helper->tpl_vars['fields_value'][$item['name']] = Configuration::get(
                    $this->settings_prefix .
                    $item['name']
                );
            }
        }

        return $helper->generateForm($field_forms);
    }

    public function getSettings()
    {
        $settings = [
            [
                'type' => 'select',
                'name' => 'FRONT_CURRENCY',
                'label' => $this->l('Front office currency'),
                'hint' => $this->l('Default front office currency. Customer can change it.'),
                'class' => 't',
                'options' => [
                    'query' => $this->getCurrencyOptions(),
                    'id' => 'id_option',
                    'name' => 'name',
                ],
                'default' => Configuration::get('PS_CURRENCY_DEFAULT'),
            ],
            [
                'type' => 'select',
                'name' => 'PS_CURRENCY_DEFAULT',
                'label' => $this->l('Back office currency'),
                'class' => 't',
                'options' => [
                    'query' => $this->getCurrencyOptions(),
                    'id' => 'id_option',
                    'name' => 'name',
                ],
                'default' => Configuration::get('PS_CURRENCY_DEFAULT'),
                'global' => true,
            ],
            [
                'type' => 'html',
                'name' => '',
                'label' => '',
                'html_content' => $this->renderManageCurrencies(),
            ],
        ];

        if ($this->getPSVersion() < 1.6) {
            foreach ($settings as &$item) {
                $desc = isset($item['desc']) ? $item['desc'] : '';
                $hint = isset($item['hint']) ? $item['hint'] . '<br/>' : '';
                $item['desc'] = $hint . $desc;
                $item['hint'] = '';
            }
        }

        return $settings;
    }

    protected function renderManageCurrencies()
    {
        $this->context->smarty->assign([
            'mc_link' => $this->context->link->getAdminLink('AdminCurrencies'),
        ]);

        return $this->context->smarty->fetch($this->local_path . 'views/templates/admin/manage_currencies.tpl');
    }

    protected function getPSVersion($without_dots = false)
    {
        $ps_version = _PS_VERSION_;
        $ps_version = substr($ps_version, 0, 3);

        if ($without_dots) {
            $ps_version = str_replace('.', '', $ps_version);
        }

        return (float) $ps_version;
    }

    protected function getCurrencyOptions()
    {
        $currencies = [];

        foreach (Currency::getCurrencies() as $currency) {
            $currencies[$currency['id_currency']] = [
                'id_option' => $currency['id_currency'],
                'name' => $currency['name'],
            ];
        }

        return $currencies;
    }

    public function refreshRates()
    {
        $this_shop = $this->context->shop->id;
        // update currency exchange rates:
        $shop_ids = Shop::getCompleteListOfShopsID();
        foreach ($shop_ids as $shop_id) {
            Shop::setContext(Shop::CONTEXT_SHOP, (int) $shop_id);
            Currency::refreshCurrencies();
        }
        Shop::setContext(Shop::CONTEXT_SHOP, (int) $this_shop);
    }

    public function refreshRatesAndPrices()
    {
        // refresh currency rates:
        $this->refreshRates();
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS(array(
                $this->_path . 'views/css/admin.css',
            ));
        }
    }
}
